==> xs-plugins-0.0.0.tgz: package/src/lib/api.js <==
/*
 * @Author: 祝玲云
 * @Date: 2021-05-19 16:02:43
 * @LastEditTime: 2021-05-21 11:11:19
 * @LastEditors: Please set LastEditors
 * @Description: \xs-plugins\src\lib\api.js
 * @FilePath: 通过项目 swagger 地址， 自动生成api.js 文件， 注册到全局， api 调用形式将变为 this.$api.cg.ChInfo()
 */

const fs = require('fs')
const path = require('path')
const http = require('http')
const https = require('https')

let apis = '' // client.js 文件， 取出 api 的 key 和 EndPoint
let serverKeyObj = {} // 端口地址和前端使用的的前缀映射关系
let apiObj = {}// 生成文件方法部分， 当前变量储存是为了对比api地址重复时， 变量会被覆盖
let keyInit = {} // this.$api.cg.ChInfo, this.$api.cg.ChInfo.Query 时， 要先给 api.cg = {} 默认值空对象 , 此变量是为了 初始化 这类变量 时比较是否已经被初始化过了, 初始化过， 追加原型属性
let num = 0 
let errNum = 0 // swagger 地址 读取失败数
let apiVariable = '' // 生成文件变量定义部分  （变量名定义， 处理 /api/cg/ChInfo  /api/cg/ChInfo/{id}, 两种形式都以 this.$api.cg.ChInfo 调用）
let apiConent = 'let api = {}\n' // 对象及对象赋值部分
class apiAutoPlugin {
    constructor(options) {
        this.options = options;
        this.output = path.join(options.rootPath , '/' + this.options.autoPath)
    }
    async apply() {
        apis = await this.getClientJS(); // 获取 clientJS 文件
        this.getApiKeyList(); // 获取 项目key 可接口端口的映射关系
        // fs.writeFile(path.join(`${this.output}`, 'client.js'), 'let apis = ' + JSON.stringify(apis) + '\n module.exports = apis;', err => { console.log(err) })  // 提供给 vue.config.js 使用 ， 处理跨域问题
        await this.getAllApi(apis ? Object.values(apis) : []) // 获取api，并书写成文件， 设置为同步， 因为 analysis.js 中 要获取 生成的 api.js 文件
    }
    // 端口地址和前端使用的qpi前缀映射关系
    getApiKeyList() {
        const obj = apis
        for (const key in obj) {
            const endPoint = obj[key]
            const keyList = endPoint.split('/')
            const serverKey = keyList[keyList.length - 2]
            serverKeyObj[serverKey] = key
        }
    }
    // 获取clientJS 文件
    getClientJS() {
        let env = this.getEnv();
        let environment = this.options.environment;
        let moduleConfig = this.options.moduleConfig;
        let baseApi = ''
        if (environment == 'env' || !environment) { // 开发环境 或者其他环境未指定 ip
            baseApi = this.formatUrl(env.VUE_APP_BASE_API + env.VUE_APP_PATH_NAME)
        } else {
            baseApi = this.formatUrl(env.VUE_APP_BASE_API.replace(/19[0-9]{1}/, environment) + env.VUE_APP_PATH_NAME)
        }
        let url = ''
        if (environment == 'environment' && env.VUE_APP_ID4_CLIENT_ID) { // clientID  开发环境取 .env.development 文件中的VUE_APP_ID4_CLIENT_ID
            const scopes = moduleConfig.DEV_CLIENT_URL.substr(moduleConfig.DEV_CLIENT_URL.indexOf('&scopes='))
            url = baseApi + '/client.js?clientID=' + this.formatUrl(env.VUE_APP_ID4_CLIENT_ID) + scopes
        } else { // 生产环境
            url = baseApi + moduleConfig.DEV_CLIENT_URL.substr(2)
        }
        const headers = { headers: { Referer: baseApi + '/' + this.formatUrl(env.VUE_APP_CLIENT_REFERER) + '/' } }
        // 根据地址获取文件
        let request = http
        if (url.substring(0, 5) == 'https') {
            request = https
        }
        return new Promise((resolve, reject) => {
            request.get(url, headers, (res) => callBack(res, resolve, reject))
        })

        function callBack(res, resolve, reject) {
            const { statusCode } = res
            let error = ''
            if (statusCode != 200) {
                error = new Error(`请求失败, 状态码: ${statusCode}, 请检查请求地址是否正确： ${url} 
                各环境项目地址请在 module.config.js 中 书写, 书写格式：
                    Compile = { 
                        PROJECT_URL: {  
                            '192': "192项目域名及端口（端口没有不写）, / 结尾" ,
                            '194': '',
                            '196': ''
                        } 
                    } `
                )
            }
            if (error) {
                console.error(error.message)
                // 消耗响应数据以释放内存
                res.resume()
                reject(false)
                return
            }
            let rawData = ''
            res.on('data', (chunk) => {
                rawData += chunk
            })
            res.on('end', () => {
                try {
                    if (rawData) {
                        const baseCfg = new Function('return ' + rawData.replace('window.$baseCfg=', ''))()
                        const apiObj = {}
                        for (const key in baseCfg.apis) {
                            apiObj[key] = baseCfg.apis[key].EndPoint
                        }
                        resolve(apiObj)
                    }
                } catch (e) {
                    reject(`错误: ${e.message}`)
                }
            })
        }
    }
    //获取 .env.development 文件 内容
    getEnv() {
        const fileUrl = path.join(`${this.options.rootPath}`, '.env.development')
        try {
            const data = fs.readFileSync(fileUrl, { encoding: 'utf8' }, err => { console.log(err) })
            const d = data.replace(/\r/g, ',').replace(/\n/g, ',') // 把换行和回车替换
            const arr = d.split(',').map(item => {
                return item.split('=')
            })
            const obj = {}
            arr.forEach(item => {
                if (item[0] && item[1]) obj[item[0].trim()] = item[1]
            })
            return obj
        } catch (error) {
            console.error(new Error('无.env.development 文件, swagger 地址无法正确获取'+fileUrl))
        }
    }
    // 主要部分
    async main(url, callBack) {
        // 获取js 文件
        const { paths, servers } = await this.httpgetJson(url)
        if (!servers) return
        const serversUrl = servers[0].url.substring(1, servers[0].url.length)
        const apiKey = serverKeyObj[serversUrl] ? serverKeyObj[serversUrl] : ''
        for (const key in paths) {
            const value = paths[key]
            let newKey = key.substr(1)
            if (newKey.indexOf(`{`) > -1) { // 先去掉参数部分， /api/cg/ChInfo  /api/cg/ChInfo/{id}, 两种形式都以 this.$api.cg.ChInfo 调用）
                newKey = newKey.substr(0, newKey.indexOf(`{`) - 1)
            }
            const variable = newKey.replace(new RegExp('/', 'g'), '') // 替换成对象形式
            const url = (apiKey + key)
            const typeKeyList = Object.keys(value) // 获取改api下有几种请求方式
            if (typeKeyList.length == 0) return
            this.isExist(typeKeyList, variable, url)
            const code = newKey.substr(4).replace(new RegExp('/', 'g'), '.') // 去掉 ap地址 上多余的 /api
            const codeObj = code.split('.') // this.$api.cg.ChInfo, this.$api.cg.ChInfo.Query 时， 要先给 api.cg = {} 默认值空对象
            codeObj.forEach((item, i) => {
                const key = this.getKey(codeObj, i)
                if (keyInit[key]) {

                } else {
                    keyInit[key] = {}
                    if (key) apiConent += `api.${key} = () => {};\n`
                }
            })
            if (keyInit[code]) {
                apiConent += `api.${code} = connect(api.${code}, ${variable});\n`
            } else {
                apiConent += `api.${code} = ${variable};\n`
                keyInit[code] = {}
            }
        }
        callBack()
    }
    // 判断变量是否重复赋值
    isExist(list, name, url) {
        if (apiObj[name]) { // 两条重复名称， 追加到函数原型的对象上
            list.forEach(type => {
                apiVariable += `${name}.${type} = (query, headers) => http.${type}('${url}', query, headers)  \n `
                apiVariable += `${name}.${type}.url = '${url}' \n`
                apiVariable += `${name}.${type}.type = '${type}' \n`
            })
        } else {
            apiVariable += `let ${name} = () => {}\n` // 一个api下 多个请求方式
            list.forEach(type => {
                if (type == 'get' || type == 'post') { // get. post 请求 默认挂再对象上
                    apiVariable += `${name} = (query, headers) => http.${type}('${url}', query, headers)\n`
                    apiVariable += `${name}.url = '${url}' \n`
                    apiVariable += `${name}.type = '${type}' \n`
                } else { // 其他请求 追加到函数原型的对象上
                    apiVariable += `${name}.${type} = (query, headers) => http.${type}('${url}', query, headers)  \n `
                    apiVariable += `${name}.${type}.url = '${url}' \n`
                    apiVariable += `${name}.${type}.type = '${type}' \n`
                }
            })
            apiObj[name] = {} // 用于对比
        }
    }

    // 获取项目相关下的所有swgger文档中的api
    getAllApi(list) {
        return new Promise((resolve)=>{
            list.forEach(item => {
                const url = item + 'Swagger/swagger.json'
                this.main(url, () => {
                    num++
                    if (num + errNum == list.length) {
                        this.writeFile()
                        resolve()
                    }
                })
            })
        })
    }
    // 根据地址获取文件
    httpgetJson(url) {
        return new Promise((resolve, reject) => {
            if (url.substring(0, 5) == 'https') {
                https.get(url, (res) => callBack(res, resolve, reject))
            } else {
                http.get(url, (res) => callBack(res, resolve, reject))
            }
        })

        function callBack(res, resolve, reject) {
            const { statusCode } = res
            const contentType = res.headers['content-type']

            let error = ''
            if (statusCode !== 200) {
                error = new Error('请求失败。\n' +
                    `状态码: ${statusCode}`)
            } else if (!/^application\/json/.test(contentType)) {
                error = new Error('无效的 content-type.\n' + `期望 application/json 但获取的是 ${contentType}`)
            }
            if (error) {
                errNum++
                console.error(error.message, url)
                // 消耗响应数据以释放内存
                res.resume()
                return
            }
            res.setEncoding('utf8')
            let rawData = ''
            res.on('data', (chunk) => {
                rawData += chunk
            })
            res.on('end', () => {
                try {
                    const parsedData = JSON.parse(rawData)
                    resolve(parsedData)
                } catch (e) {
                    reject(`错误: ${e.message}`)
                }
            })
        }
    }
    // 获取 api 的key
    getKey(list, index) {
        let res = ''
        for (let i = 0; i < index; i++) {
            res = res + list[i] + '.'
        }
        return res.substr(0, res.length - 1)
    }
    // 生成文件
    writeFile() {
        fs.writeFile(path.join(`${this.output}`, 'api.js'), this.generateTemplate() + this.getFun() + apiVariable + apiConent + '\n' + this.exportTemplate(), err => { console.log(err) })
    }
    formatUrl(url) {
        if (url) return url.replace(new RegExp(/['"\s]/, 'g'), '')
    }
    // 导入头部引入文件
    generateTemplate() {
        return `import http from "../../src/plugins/request/request.js";\n`
    }
    // 导出
    exportTemplate() {
        return `export default api`
    }
    // 处理数据
    getFun() {
        return `
        function connect (src, append) {
            for (const key in src) {
                append[key] = src[key]
            }
            return append
        }
        `
    }
    getHttp() {
        return `let http = {
            post: ()=> {},
            get: () => {},
            put: () => {},
            delete: () => {}
        }`
    }
    getApi () {
        let aApi = eval(this.getHttp() + this.getFun() + apiVariable + apiConent + '\n' + 'module.exports = api')
        return aApi
    }
}

module.exports = apiAutoPlugin

==> xs-plugins-0.0.3.tgz: package/src/lib/api.js <==
/*
 * @Author: 祝玲云
 * @Date: 2021-05-19 16:02:43
 * @LastEditTime: 2021-05-21 15:13:49
 * @LastEditors: Please set LastEditors
 * @Description: \xs-plugins\src\lib\api.js
 * @FilePath: 通过项目 swagger 地址， 自动生成api.js 文件， 注册到全局， api 调用形式将变为 this.$api.cg.ChInfo()
 */

const fs = require('fs')
const path = require('path')
const http = require('http')
const https = require('https')

let apis = '' // client.js 文件， 取出 api 的 key 和 EndPoint
let serverKeyObj = {} // 端口地址和前端使用的的前缀映射关系
let apiObj = {}// 生成文件方法部分， 当前变量储存是为了对比api地址重复时， 变量会被覆盖
let keyInit = {} // this.$api.cg.ChInfo, this.$api.cg.ChInfo.Query 时， 要先给 api.cg = {} 默认值空对象 , 此变量是为了 初始化 这类变量 时比较是否已经被初始化过了, 初始化过， 追加原型属性
let num = 0 
let errNum = 0 // swagger 地址 读取失败数
let apiVariable = '' // 生成文件变量定义部分  （变量名定义， 处理 /api/cg/ChInfo  /api/cg/ChInfo/{id}, 两种形式都以 this.$api.cg.ChInfo 调用）
let apiConent = 'let api = {}\n' // 对象及对象赋值部分
class ApiPlugin {
    constructor(options) {
        this.options = options;
        this.output = path.join(options.rootPath , '/' + this.options.autoPath)
    }
    async apply() {
        apis = await this.getClientJS(); // 获取 clientJS 文件
        this.getApiKeyList(); // 获取 项目key 可接口端口的映射关系
        await this.getAllApi(apis ? Object.values(apis) : []) // 获取api，并书写成文件， 设置为同步， 因为 analysis.js 中 要获取 生成的 api.js 文件
    }
    // 端口地址和前端使用的qpi前缀映射关系
    getApiKeyList() {
        const obj = apis
        for (const key in obj) {
            const endPoint = obj[key]
            const keyList = endPoint.split('/')
            const serverKey = keyList[keyList.length - 2]
            serverKeyObj[serverKey] = key
        }
    }
    // 获取clientJS 文件
    getClientJS() {
        let env = this.getEnv();
        let environment = this.options.environment;
        let moduleConfig = this.options.moduleConfig;
        let baseApi = ''
        if (environment == 'env' || !environment) { // 开发环境 或者其他环境未指定 ip
            baseApi = this.formatUrl(env.VUE_APP_BASE_API + env.VUE_APP_PATH_NAME)
        } else { //生产环境
            let str = /^19[0-9]{1}$/
            if(str === 'pro') { str = 192 } // 正式环境取192 swagger 地址
            environment = environment.trim()
            if(str.test(environment)){
              if(modelConfig.PROJECT_URL && modelConfig.PROJECT_URL[environment]) {
                baseApi = formatUrl(modelConfig.PROJECT_URL[environment] + env.VUE_APP_PATH_NAME)
              } else {
                baseApi = formatUrl(env.VUE_APP_BASE_API.replace(/19[0-9]{1}/, environment) + env.VUE_APP_PATH_NAME)
              }
            }
        } 
        let url = ''
        if (environment == 'environment' && env.VUE_APP_ID4_CLIENT_ID) { // clientID  开发环境取 .env.development 文件中的VUE_APP_ID4_CLIENT_ID
            const scopes = moduleConfig.DEV_CLIENT_URL.substr(moduleConfig.DEV_CLIENT_URL.indexOf('&scopes='))
            url = baseApi + '/client.js?clientID=' + this.formatUrl(env.VUE_APP_ID4_CLIENT_ID) + scopes
        } else { // 生产环境
            url = baseApi + moduleConfig.DEV_CLIENT_URL.substr(2)
        }
        const headers = { headers: { Referer: baseApi + '/' + this.formatUrl(env.VUE_APP_CLIENT_REFERER) + '/' } }
        // 根据地址获取文件
        let request = http
        if (url.substring(0, 5) == 'https') {
            request = https
        }
        return new Promise((resolve, reject) => {
            request.get(url, headers, (res) => callBack(res, resolve, reject))
        })

        function callBack(res, resolve, reject) {
            const { statusCode } = res
            let error = ''
            if (statusCode != 200) {
                error = new Error(`请求失败, 状态码: ${statusCode}, 请检查请求地址是否正确： ${url} 
                各环境项目地址请在 module.config.js 中 书写, 书写格式：
                    Compile = { 
                        PROJECT_URL: {  
                            '192': "192项目域名及端口（端口没有不写）, / 结尾" ,
                            '194': '',
                            '196': ''
                        } 
                    } `
                )
            }
            if (error) {
                console.error(error.message)
                // 消耗响应数据以释放内存
                res.resume()
                reject(false)
                return
            }
            let rawData = ''
            res.on('data', (chunk) => {
                rawData += chunk
            })
            res.on('end', () => {
                try {
                    if (rawData) {
                        const baseCfg = new Function('return ' + rawData.replace('window.$baseCfg=', ''))()
                        const apiObj = {}
                        for (const key in baseCfg.apis) {
                            apiObj[key] = baseCfg.apis[key].EndPoint
                        }
                        resolve(apiObj)
                    }
                } catch (e) {
                    reject(`错误: ${e.message}`)
                }
            })
        }
    }
    //获取 .env.development 文件 内容
    getEnv() {
        const fileUrl = path.join(`${this.options.rootPath}`, '.env.development')
        try {
            const data = fs.readFileSync(fileUrl, { encoding: 'utf8' }, err => { console.log(err) })
            const d = data.replace(/\r/g, ',').replace(/\n/g, ',') // 把换行和回车替换
            const arr = d.split(',').map(item => {
                return item.split('=')
            })
            const obj = {}
            arr.forEach(item => {
                if (item[0] && item[1]) obj[item[0].trim()] = item[1]
            })
            return obj
        } catch (error) {
            console.error(new Error('无.env.development 文件, swagger 地址无法正确获取'+fileUrl))
        }
    }
    // 主要部分
    async main(url, callBack) {
        // 获取js 文件
        const { paths, servers } = await this.httpgetJson(url)
        if (!servers) return
        const serversUrl = servers[0].url.substring(1, servers[0].url.length)
        const apiKey = serverKeyObj[serversUrl] ? serverKeyObj[serversUrl] : ''
        for (const key in paths) {
            const value = paths[key]
            let newKey = key.substr(1)
            if (newKey.indexOf(`{`) > -1) { // 先去掉参数部分， /api/cg/ChInfo  /api/cg/ChInfo/{id}, 两种形式都以 this.$api.cg.ChInfo 调用）
                newKey = newKey.substr(0, newKey.indexOf(`{`) - 1)
            }
            const variable = newKey.replace(new RegExp('/', 'g'), '') // 替换成对象形式
            const url = (apiKey + key)
            const typeKeyList = Object.keys(value) // 获取改api下有几种请求方式
            if (typeKeyList.length == 0) return
            this.isExist(typeKeyList, variable, url)
            const code = newKey.substr(4).replace(new RegExp('/', 'g'), '.') // 去掉 ap地址 上多余的 /api
            const codeObj = code.split('.') // this.$api.cg.ChInfo, this.$api.cg.ChInfo.Query 时， 要先给 api.cg = {} 默认值空对象
            codeObj.forEach((item, i) => {
                const key = this.getKey(codeObj, i)
                if (keyInit[key]) {

                } else {
                    keyInit[key] = {}
                    if (key) apiConent += `api.${key} = () => {};\n`
                }
            })
            if (keyInit[code]) {
                apiConent += `api.${code} = connect(api.${code}, ${variable});\n`
            } else {
                apiConent += `api.${code} = ${variable};\n`
                keyInit[code] = {}
            }
        }
        callBack()
    }
    // 判断变量是否重复赋值
    isExist(list, name, url) {
        if (apiObj[name]) { // 两条重复名称， 追加到函数原型的对象上
            list.forEach(type => {
                apiVariable += `${name}.${type} = (query, headers) => http.${type}('${url}', query, headers)  \n `
                apiVariable += `${name}.${type}.url = '${url}' \n`
                apiVariable += `${name}.${type}.type = '${type}' \n`
            })
        } else {
            apiVariable += `let ${name} = () => {}\n` // 一个api下 多个请求方式
            list.forEach(type => {
                if (type == 'get' || type == 'post') { // get. post 请求 默认挂再对象上
                    apiVariable += `${name} = (query, headers) => http.${type}('${url}', query, headers)\n`
                    apiVariable += `${name}.url = '${url}' \n`
                    apiVariable += `${name}.type = '${type}' \n`
                } else { // 其他请求 追加到函数原型的对象上
                    apiVariable += `${name}.${type} = (query, headers) => http.${type}('${url}', query, headers)  \n `
                    apiVariable += `${name}.${type}.url = '${url}' \n`
                    apiVariable += `${name}.${type}.type = '${type}' \n`
                }
            })
            apiObj[name] = {} // 用于对比
        }
    }

    // 获取项目相关下的所有swgger文档中的api
    getAllApi(list) {
        return new Promise((resolve)=>{
            list.forEach(item => {
                const url = item + 'Swagger/swagger.json'
                this.main(url, () => {
                    num++
                    if (num + errNum == list.length) {
                        this.writeFile()
                        resolve()
                    }
                })
            })
        })
    }
    // 根据地址获取文件
    httpgetJson(url) {
        return new Promise((resolve, reject) => {
            if (url.substring(0, 5) == 'https') {
                https.get(url, (res) => callBack(res, resolve, reject))
            } else {
                http.get(url, (res) => callBack(res, resolve, reject))
            }
        })

        function callBack(res, resolve, reject) {
            const { statusCode } = res
            const contentType = res.headers['content-type']

            let error = ''
            if (statusCode !== 200) {
                error = new Error('请求失败。\n' +
                    `状态码: ${statusCode}`)
            } else if (!/^application\/json/.test(contentType)) {
                error = new Error('无效的 content-type.\n' + `期望 application/json 但获取的是 ${contentType}`)
            }
            if (error) {
                errNum++
                console.error(error.message, url)
                // 消耗响应数据以释放内存
                res.resume()
                return
            }
            res.setEncoding('utf8')
            let rawData = ''
            res.on('data', (chunk) => {
                rawData += chunk
            })
            res.on('end', () => {
                try {
                    const parsedData = JSON.parse(rawData)
                    resolve(parsedData)
                } catch (e) {
                    reject(`错误: ${e.message}`)
                }
            })
        }
    }
    // 获取 api 的key
    getKey(list, index) {
        let res = ''
        for (let i = 0; i < index; i++) {
            res = res + list[i] + '.'
        }
        return res.substr(0, res.length - 1)
    }
    // 生成文件
    writeFile() {
        fs.writeFile(path.join(`${this.output}`, 'api.js'), this.generateTemplate() + this.getFun() + apiVariable + apiConent + '\n' + this.exportTemplate(), err => { console.log(err) })
    }
    formatUrl(url) {
        if (url) return url.replace(new RegExp(/['"\s]/, 'g'), '')
    }
    // 导入头部引入文件
    generateTemplate() {
        return `import http from "../../src/plugins/request/request.js";\n`
    }
    // 导出
    exportTemplate() {
        return `export default api`
    }
    // 处理数据
    getFun() {
        return `
        function connect (src, append) {
            for (const key in src) {
                append[key] = src[key]
            }
            return append
        }
        `
    }
    getHttp() {
        return `let http = {
            post: ()=> {},
            get: () => {},
            put: () => {},
            delete: () => {}
        }`
    }
    getApi () {
        let aApi = eval(this.getHttp() + this.getFun() + apiVariable + apiConent + '\n' + 'module.exports = api')
        return aApi
    }
}

module.exports = ApiPlugin
